package fxLangat;

import java.awt.Desktop;
import java.awt.Label;


import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import langat.Kera;
import langat.Kohteet;
import langat.Langat;
import langat.SailoException;

/**
 * @author Essi
 * @version 2.2.2025
 *
 */


public class LangatGUIController implements Initializable { 
    
    @FXML private TextField hakuehto;
    @FXML private Label labelVirhe;
    @FXML private ScrollPane panelKera;
    @FXML private ListChooser<Kera> chooserKerat;
    
    @FXML private ComboBoxChooser<?> editVari;
    @FXML private ComboBoxChooser<?> editMateriaali;
    @FXML private ComboBoxChooser<?> editKaytto;
    @FXML private TextField editMaara;
    
    @FXML private ComboBoxChooser<?> cbVari;
    @FXML private ComboBoxChooser<?> cbMateriaali;
    @FXML private ComboBoxChooser<?> cbKaytto;
    
    @FXML private TextArea areaHyodynnys;
    
    @Override
    public void initialize(URL url, ResourceBundle bundle) {
        alusta();
    }


    @FXML
    void handleLopeta() {
        lopeta();
    }

    @FXML
    void handleMuokkaa() {
        muokkaa();        
    }

    @FXML
    void handlePoista() {
        boolean vastaus = Dialogs.showQuestionDialog("Poisto?",
                "Haluatko varmasti poistaa tiedot?", "Kyllä", "Ei");
        if ( vastaus==true ) poista();      
    }

    @FXML
    void handleTallenna() {
        tallenna();
    }
    
    @FXML
    void handleOhje() {
        avustus();
    }

    @FXML
    void handleUusi() {
        uusiKera();
    }
    

//=========================================================================
    
    private String rekisterinnimi = "langat";
    private Langat langat;
    private Kera keraKohdalla;
    private ComboBoxChooser<?> edits[];
    
    
    private String tallenna() {
        try {
            langat.tallenna();
            return null;
        } catch (SailoException ex) {
            Dialogs.showMessageDialog("Tallennuksessa ongelmia! " + ex.getMessage());
            return ex.getMessage();
        }
    }

    
    private void lopeta() {
        tallenna();
        Platform.exit();
    }
    
    private void poista() {
        langat.poistaKera(chooserKerat.getSelectedIndex());
        langat.poistaKohteet(keraKohdalla.getTunnusNro());
        hae(0);
    }
    
    private void muokkaa() {
         if ( keraKohdalla == null ) return; 
         try { 
             Kera kera= LanganmuokkausController.kysyKera(null, keraKohdalla.clone(),langat,this); 
             if ( kera == null ) return; 
             langat.korvaaTaiLisaa(kera); 
             hae(kera.getTunnusNro()); 
         } catch (CloneNotSupportedException e) { 
                      // 
         } catch (SailoException e) { 
              Dialogs.showMessageDialog(e.getMessage()); 
              } 
         } 
        
    
    
    private void avustus() {
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/v/2025/kevat/ht/essmarpe");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
        
    }
    
    /**
     * @param langat kerhonnimi
     */
    public void setKerho(Langat langat) {
           this.langat = langat;
           naytaKera();
           lueTiedosto(rekisterinnimi);
    }
    
    /**
     * @param nimi rekisterin nimi
     * @return virhe
     */
    protected String lueTiedosto(String nimi) {
        rekisterinnimi = nimi;

        try {
            langat.lueTiedostosta(nimi);
            hae(0);
            return null;
        } catch (SailoException e) {
            hae(0);
            String virhe = e.getMessage(); 
            if ( virhe != null ) Dialogs.showMessageDialog(virhe);
            return virhe;
        }
     }

    
    /**
     * Tekee tarvittavat muut alustukset
     */
    protected void alusta() {
        chooserKerat.clear();
        chooserKerat.addSelectionListener(e -> naytaKera());
        
        edits = new ComboBoxChooser<?>[]{editVari, editMateriaali, editKaytto};
        
        cbVari.getSelectionModel().selectedIndexProperty().addListener((obs, oldVal, newVal) -> {hae(0);});
        cbMateriaali.getSelectionModel().selectedIndexProperty().addListener((obs, oldVal, newVal) -> {hae(0);});;
        cbKaytto.getSelectionModel().selectedIndexProperty().addListener((obs, oldVal, newVal) -> {hae(0);});
    }
    

    
    /**
     * Näyttää listasta valitun kerän tiedot
     */
    protected void naytaKera() {
        keraKohdalla = chooserKerat.getSelectedObject();

        if (keraKohdalla == null) {
            areaHyodynnys.clear();
            return;
        }
        edits[0].getSelectionModel().select(keraKohdalla.getVariInd());
        edits[1].getSelectionModel().select(keraKohdalla.getMateriaaliInd());
        edits[2].getSelectionModel().select(keraKohdalla.getKayttoInd());
        editMaara.setText(keraKohdalla.getMaara()+"");
        areaHyodynnys.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaHyodynnys)) {
            tulosta(os, keraKohdalla);
        }
        
    }
    

    
    /**
     * Hakee kerien tiedot listaan
     * @param jnro kerän numero, joka aktivoidaan haun jälkeen
     */ 
    protected void hae(int jnro) {
        int jnr =jnro;
        chooserKerat.clear();

        int kv = cbVari.getSelectionModel().getSelectedIndex();
        int km = cbMateriaali.getSelectionModel().getSelectedIndex();
        int kk = cbKaytto.getSelectionModel().getSelectedIndex();
        
        
        
        int index = 0;
        for (int i = 0; i < langat.getJasenia(); i++) {
            Kera kera = langat.annaJasen(i);
            if (kera.getTunnusNro() == jnr) index = i;
            boolean matches = 
                    (kv == 0 || kera.getVariInd() == kv) &&
                    (km == 0 || kera.getMateriaaliInd() == km) &&
                    (kk == 0 || kera.getKayttoInd() == kk);

                if (matches ) {
                    chooserKerat.add(kera.toString(kera), kera);
        }
        
        chooserKerat.setSelectedIndex(index); 
    }
    }
    


    
    /**
     * Luo uuden kerän jota aletaan editoimaan 
     */
    protected void uusiKera() {
        try {
            Kera uusi = new Kera();
            uusi.rekisteroi();
            uusi = LanganmuokkausController.kysyKera(null, uusi,langat,this);             
            if ( uusi == null ) return;            
            
            langat.lisaa(uusi);
            hae(uusi.getTunnusNro());

        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
            return;
        }
    }
    
    /**
     * @param os tietovirta
     * @param kera lanka
     */
    public void tulosta(PrintStream os, final Kera kera) {
        List<Kohteet> hyodynnys = langat.annaHyodynnysKohteet(kera);  
        for (Kohteet kohd:hyodynnys)
            kohd.tulosta(os);  
        }
    

}
