package langat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;

/**
 * @author Käyttäjä
 * @version 6.4.2025
 *
 */
public class Hyodynnys implements Iterable<Kohteet>{

    //private String                      tiedostonNimi = "";

    /** Taulukko käyttökohteista */
    private final Collection<Kohteet> alkiot        = new ArrayList<Kohteet>();

    
    /**
     * Lisää uuden käyttökohteen tietorakenteeseen.  Ottaa sen omistukseensa.
     * @param kohd lisättävä kohde
     */
    public void lisaa(Kohteet kohd) {
        alkiot.add(kohd);
    }

    /**
     * Poistaa kohteen
     * @param kohd poistettava kohde
     */
    public void poista(Kohteet kohd) {
        Iterator<Kohteet> iteraa = alkiot.iterator();
        while (iteraa.hasNext()) {
            Kohteet k = iteraa.next();
            if (kohd.toString(kohd).equals(k.toString(k))) {
                iteraa.remove();
                
            }
        }
        return;
    }
    

    /**
     * Poistetaan kohteet kerältä
     * @param paikka kerän id
     */
    public void poistaKohteet(int paikka) {
        Iterator<Kohteet> iteraa = alkiot.iterator();
        while (iteraa.hasNext()) {
            Kohteet k = iteraa.next();
            if (k.getKeraNro()==paikka) {
                iteraa.remove();
            }
        }
    }


    /**
     * Haetaan kaikki jäsen käyttökohteet
     * @param tunnusnro jäsenen tunnusnumero jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Hyodynnys missavoi = new Hyodynnys();
     *  Kohteet pipo21 = new Kohteet(2); missavoi.lisaa(pipo21);
     *  Kohteet pipo11 = new Kohteet(1); missavoi.lisaa(pipo11);
     *  Kohteet pipo22 = new Kohteet(2); missavoi.lisaa(pipo22);
     *  Kohteet pipo12 = new Kohteet(1); missavoi.lisaa(pipo12);
     *  Kohteet pipo23 = new Kohteet(2); missavoi.lisaa(pipo23);
     *  Kohteet pipo51 = new Kohteet(5); missavoi.lisaa(pipo51);
     *  
     *  
     *  List<Kohteet> loytyneet;
     *  loytyneet = missavoi.annaHyodynnysKohteet(3);
     *  loytyneet.size() === 0; 
     *  loytyneet = missavoi.annaHyodynnysKohteet(1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pipo11 === true;
     *  loytyneet.get(1) == pipo12 === true;
     *  loytyneet = missavoi.annaHyodynnysKohteet(5);
     *  loytyneet.size() === 1; 
     *  loytyneet.get(0) == pipo51 === true;
     * </pre> 
     */
    public List<Kohteet> annaHyodynnysKohteet(int tunnusnro) {
        List<Kohteet> loydetyt = new ArrayList<Kohteet>();
        for (Kohteet kohd : alkiot)
            if (kohd.getKeraNro() == tunnusnro) loydetyt.add(kohd);
        return loydetyt;
    }
    

    
  
    
    /**
     * 
     * @param hakemisto hakemisto johon luodaan
     * @throws SailoException jos lukeminen epäonnistuu
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.File;
     *  Hyodynnys hyod = new Hyodynnys();
     *  Kohteet pitsi21 = new Kohteet(); pitsi21.taytaPipo(2);
     *  Kohteet pitsi11 = new Kohteet(); pitsi11.taytaPipo(1);
     *  Kohteet pitsi22 = new Kohteet(); pitsi22.taytaPipo(2);
     *  Kohteet pitsi12 = new Kohteet(); pitsi12.taytaPipo(1); 
     *  Kohteet pitsi23 = new Kohteet(); pitsi23.taytaPipo(2); 
     *  String tiedNimi = "testikelmit";
     *  File ftied = new File(tiedNimi+".dat");
     *  ftied.delete();
     *  hyod.lueTiedostosta(tiedNimi); 
     *  hyod.lisaa(pitsi21);
     *  hyod.lisaa(pitsi11);
     *  hyod.lisaa(pitsi22);
     *  hyod.lisaa(pitsi12);
     *  hyod.lisaa(pitsi23);
     *  hyod.tallenna(tiedNimi);
     *  hyod = new Hyodynnys();
     *  hyod.lueTiedostosta(tiedNimi); 
     *  Iterator<Kohteet> i = hyod.iterator();
     *  i.next().toString() === pitsi21.toString();
     *  i.next().toString() === pitsi11.toString();
     *  i.next().toString() === pitsi22.toString();
     *  i.next().toString() === pitsi12.toString();
     *  i.next().toString() === pitsi23.toString();
     *  i.hasNext() === true;
     *  hyod.lisaa(pitsi23);
     *  hyod.tallenna(tiedNimi); 
     *  ftied.delete() === false;
     *  File fbak = new File(tiedNimi+".bak");
     *  fbak.delete() === false;
     * </pre>
     */

    public void lueTiedostosta(String hakemisto) throws SailoException {
        String nimi = hakemisto + "/missavoihyodyntaa.dat";
        File ftied = new File(nimi);
        try (Scanner fi = new Scanner(new FileInputStream(ftied))) { // Jotta UTF8/ISO-8859 toimii'
            while ( fi.hasNext() ) {
                String s = fi.nextLine().trim();
                if ( "".equals(s) || s.charAt(0) == ';' ) continue;
                Kohteet kohd = new Kohteet();
                kohd.parse(s);
                lisaa(kohd);
            }
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Ei saa luettua tiedostoa " + nimi);
        }
    }

    
    /**
     * Tallentaa jäsenistön tiedostoon.  
     * Tiedoston muoto:
     * <pre>
     * 2|Ankka Aku|121103-706Y|Paratiisitie 13|12345|ANKKALINNA|12-1234|||1996|50.0|30.0|Velkaa Roopelle
     * 3|Ankka Tupu|121153-706Y|Paratiisitie 13|12345|ANKKALINNA|12-1234|||1996|50.0|30.0|Velkaa Roopelle
     * </pre>
     * @param hakemisto tallennettavan tiedoston hakemisto
     * @throws SailoException jos talletus epäonnistuu
     */
    public void tallenna(String hakemisto) throws SailoException {
        File ftied = new File(hakemisto + "/missavoihyodyntaa.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(ftied, false))) {
            for (var kohd: alkiot) {
                fo.println(kohd.toString());
            }
        } catch (FileNotFoundException ex) {
            throw new SailoException("Tiedosto " + ftied.getAbsolutePath() + " ei aukea");
        }        
    }


    
    /**
     * Testiohjelma harrastuksille
     * @param args ei käytössä
     */
    public static void main(String[] args) {

        Hyodynnys missavoi = new Hyodynnys();
        
        try {
            missavoi.lueTiedostosta("langat");
        } catch (SailoException ex) {
            System.err.println(ex.getMessage());
        }
        
        Kohteet pipo1 = new Kohteet();
        pipo1.taytaPipo(2);
        pipo1.rekisteroi();
        Kohteet pipo2 = new Kohteet();
        pipo2.taytaPipo(1);
        pipo2.rekisteroi();
        Kohteet pipo3 = new Kohteet();
        pipo3.taytaPipo(2);
        pipo3.rekisteroi();
        Kohteet pipo4 = new Kohteet();
        pipo4.taytaPipo(2);
        pipo4.rekisteroi();

        missavoi.lisaa(pipo1);
        missavoi.lisaa(pipo2);
        missavoi.lisaa(pipo3);
        missavoi.lisaa(pipo4);

        System.out.println("============= Kohteet testi =================");

        List<Kohteet> hyodynnys2 = missavoi.annaHyodynnysKohteet(2);

        for (Kohteet kohd : hyodynnys2) {
            System.out.print(kohd.getKeraNro() + " ");
            kohd.tulosta(System.out);
        }
        
        try {
            missavoi.tallenna("langat");
        } catch (SailoException e) {
            e.printStackTrace();
        }


    }
    
    /**
     * Iteraattori kaikkien harrastusten läpikäymiseen
     * @return harrastusiteraattori
     * 
     * @example
     * <pre name="test">
     * #PACKAGEIMPORT
     * #import java.util.*;
     * 
     *  Hyodynnys kohteit = new Hyodynnys();
     *  Kohteet pipo21 = new Kohteet(2); kohteit.lisaa(pipo21);
     *  Kohteet pipo11 = new Kohteet(1); kohteit.lisaa(pipo11);
     *  Kohteet pipo22 = new Kohteet(2); kohteit.lisaa(pipo22);
     *  Kohteet pipo12 = new Kohteet(1); kohteit.lisaa(pipo12);
     *  Kohteet pipo23 = new Kohteet(2); kohteit.lisaa(pipo23);
     * 
     *  Iterator<Kohteet> i2=kohteit.iterator();
     *  i2.next() === pipo21;
     *  i2.next() === pipo11;
     *  i2.next() === pipo22;
     *  i2.next() === pipo12;
     *  i2.next() === pipo23;
     *  i2.next() === pipo12;  #THROWS NoSuchElementException  
     *  
     *  int n = 0;
     *  int jnrot[] = {2,1,2,1,2};
     *  
     *  for ( Kohteet kohd:kohteit ) { 
     *    kohd.getKeraNro() === jnrot[n]; n++;  
     *  }
     *  
     *  n === 5;
     *  
     * </pre>
     */
    @Override
    public Iterator<Kohteet> iterator() {
        return alkiot.iterator();
    }



}
