/**
 * 
 */
package langat;

import java.io.OutputStream;
import java.io.PrintStream;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author Käyttäjä
 * @version 4.3.2025
 *
 */
public class Kera implements Cloneable {
    
    private int        tunnusNro = 0;
    private int        maara = 0;

    private int variInd = 0;
    private int materiaaliInd = 0;
    private int kayttoInd = 0;
    
    private static int seuraavaNro    = 1;

    

    
    /**
     * tulostaa lankojen tideot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", tunnusNro, 3) + "  " + variInd + "  "
                + materiaaliInd + "  " + kayttoInd + "  " + maara);
       

    }
    /**
     * Tulostetaan langan tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    
    /**
     * Antaa kerälle seuraavan rekisterinumeron.
     * @return jäsenen uusi tunnusNro
     * @example
     * <pre name="test">
     *   Kera lanka1 = new Kera();
     *   lanka1.getTunnusNro() === 0;
     *   lanka1.rekisteroi();
     *   Kera lanka2 = new Kera();
     *   lanka2.rekisteroi();
     *   int n1 = lanka1.getTunnusNro();
     *   int n2 = lanka2.getTunnusNro();
     *   n1 === n2-1;
     * </pre>
     */
    public int rekisteroi() {
        tunnusNro = seuraavaNro;
        seuraavaNro++;
        return tunnusNro;
    }


    /**
     * Palauttaa kerän tunnusnumeron.
     * @return kerän tunnusnumero
     */
    public int getTunnusNro() {
        return tunnusNro;
    }

    
    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot kerälle.
     */
    public void tayta() {
        maara = 4;
        
        variInd = 1;
        materiaaliInd = 1;
        kayttoInd = 1;
    }
    
    /**
     * Tulostaa keran oikeassa muodossa listaan
     * @param kera kera joka tulostetaan
     * @return String oikeassa muodossa
     */
    public String toString(Kera kera) {
        String[] varit = {"","Valkoinen", "Sininen", "Punainen", "Musta", "Pinkki","Keltainen", "Vihreä"};

            String[] materiaalit = {"","Villa", "Puuvilla", "Polyesteri"};

            String[] kaytot = {"","Virkkaus/kudonta","Ompelu"};

            String varilist = (kera.getVariInd() > 0 && kera.getVariInd() < varit.length)
                          ? varit[kera.getVariInd()] : "Valkoinen";

            String materiaalilist = (kera.getMateriaaliInd() > 0 && kera.getMateriaaliInd() < materiaalit.length)
                                ? materiaalit[kera.getMateriaaliInd()] : "Villa";

            String kayttolist = (kera.getKayttoInd() > 0 && kera.getKayttoInd() < kaytot.length)
                            ? kaytot[kera.getKayttoInd()] : "Ompelu";

            return  varilist + "  "
                   + materiaalilist + "  " + kayttolist + "  " + maara;
        }

    
    @Override
    public String toString() {
        return "" +
                getTunnusNro() + "|" +
                variInd + "|" +
                materiaaliInd + "|" +
                kayttoInd + "|" +
                maara ;
    }

    

    /**
     * @param rivi parsittava rivi
     */
    public void parse(String rivi) {
        StringBuilder sb = new StringBuilder(rivi);
        setTunnusNro(Mjonot.erota(sb, '|', getTunnusNro()));

        variInd = Mjonot.erota(sb, '|', variInd);
        materiaaliInd = Mjonot.erota(sb, '|', materiaaliInd);
        kayttoInd = Mjonot.erota(sb, '|', kayttoInd);
        maara = Mjonot.erota(sb, '|', maara);
    }
    
    /**
     * Asettaa tunnusnumeron ja samalla varmistaa että
     * seuraava numero on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava tunnusnumero
     */
    private void setTunnusNro(int nr) {
        tunnusNro = nr;
        if (tunnusNro >= seuraavaNro) seuraavaNro = tunnusNro + 1;
    }

    @Override
    public boolean equals(Object kera) {
        if ( kera == null ) return false;
        return this.toString().equals(kera.toString());
    }


    @Override
    public int hashCode() {
        return tunnusNro;
    }
    
    /**
     * Tehdään identtinen klooni jäsenestä
     * @return Object kloonattu jäsen
     * @example
     * <pre name="test">
     * #THROWS CloneNotSupportedException 
     *   Kera kera = new Kera();
     *   kera.parse("   3  |  Ankka Aku   | 123");
     *   Kera kopio = kera.clone();
     *   kopio.toString() === kera.toString();
     *   kera.parse("   4  |  Ankka Tupu   | 123");
     *   kopio.toString().equals(kera.toString()) === false;
     * </pre>
     */
    @Override
    public Kera clone() throws CloneNotSupportedException {
        Kera uusi;
        uusi = (Kera) super.clone();
        return uusi;
    }

    /**
     * @param s määrä
     * @return virheen
     * @throws NumberFormatException jos ongelmia
     */
    public String setMaara(String s) throws NumberFormatException{
        try {
            maara = Integer.valueOf(s);
            return null; 
        } catch (NumberFormatException e) {
            return "Syötä määrä numeroina";
        }
    }
    
    /**
     * asettaa värin
     * @param ind värin indeksi
     */
    public void setVariInd(int ind) {
        this.variInd = ind;
    }

    /**
     * asettaa materiaalin
     * @param ind materiaalin indeksi
     */
    public void setMateriaaliInd(int ind) {
        this.materiaaliInd = ind;
    }
    /**
     * asettaa käyttökohteen
     * @param ind käyttökohteen indeksi
     */
    public void setKayttoiInd(int ind) {
        this.kayttoInd = ind;
    }
    
    /**
     * @return kerien määrä
     */
    public int getMaara() {
        return maara;
    }
    
    /**
     * @return värin indeksi
     */
    public int getVariInd() {
        return variInd;
    }
    /**
     * @return käyttökohteen indeksi
     */
    public int getKayttoInd() {
        return kayttoInd;
    }
    /**
     * @return materiaalin indeksi
     */
    public int getMateriaaliInd() {
        return materiaaliInd;
    }
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Kera lanka = new Kera();
        Kera lanka2 = new Kera();
        
        lanka.rekisteroi();
        lanka2.rekisteroi();
        
        lanka.tulosta(System.out);
        lanka.tayta();
        lanka.tulosta(System.out);

        lanka2.tayta();
        lanka2.tulosta(System.out);

        lanka2.tayta();
        lanka2.tulosta(System.out);

    
    }
    
   

}
