package langat;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
//import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Scanner;

/**
 * @author Käyttäjä
 * @version 31.3.2025
 *
 */
public class Kerat implements Iterable<Kera>{

    private static final int MAX_JASENIA   = 1;
    @SuppressWarnings("unused")
    private boolean muutettu = false;
    private int lkm = 0;
    private String tiedostonPerusNimi = "nimet";

    private Kera            alkiot[];

    /**
     * Oletusmuodostaja
     */
    public Kerat() {
        this.alkiot = new Kera[MAX_JASENIA];
    }

    /**
     * Lisää uuden jäsenen tietorakenteeseen.  Ottaa jäsenen omistukseensa.
     * @param kera lisättävä kerä
     * @throws SailoException jos tietorakenne on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Kerat kerat = new Kerat();
     * Kera lanka1 = new Kera(), lanka2 = new Kera();
     * kerat.getLkm() === 0;
     * kerat.lisaa(lanka1); kerat.getLkm() === 1;
     * kerat.lisaa(lanka2); kerat.getLkm() === 2;
     * kerat.lisaa(lanka1); kerat.getLkm() === 3;
     * kerat.anna(0) === lanka1;
     * kerat.anna(1) === lanka2;
     * kerat.anna(2) === lanka1;
     * kerat.anna(1) == lanka1 === false;
     * kerat.anna(1) == lanka2 === true;
     * kerat.lisaa(lanka1); kerat.getLkm() === 4;
     * kerat.lisaa(lanka1); kerat.getLkm() === 5;
     * kerat.lisaa(lanka1);  
     * </pre>
     */
    public void lisaa(Kera kera) throws SailoException {
        if (lkm >= alkiot.length) alkiot = Arrays.copyOf(alkiot, alkiot.length+10);;
        this.alkiot[this.lkm] = kera;
        this.lkm++;
        muutettu=true;
    }
    

    /** poistetaan kerä
     * @param paikka poistettavan kerän id
     */
    public void poistaKera(int paikka) {
        for (int i = paikka; i < lkm-1; i++) {
            alkiot[i] = alkiot[i+1]; 
        }
        alkiot[lkm-1] = null; 
        lkm--;
        muutettu = true;
      
    }
    /**
     * Korvaa kerär tietorakenteessa.
     * Etsitään samalla tunnusnumerolla oleva kerä.  Jos ei löydy,
     * niin lisätään uutena keränä.
     * @param kera kera joka uodaan
     * @throws SailoException jos tietorakenne on jo täynnä
     * <pre name="test">
     * #THROWS SailoException,CloneNotSupportedException
     * #PACKAGEIMPORT
     * Kerat kerat = new Kerat();
     * Kera aku1 = new Kera(), aku2 = new Kera();
     * aku1.rekisteroi(); aku2.rekisteroi();
     * kerat.getLkm() === 0;
     * kerat.korvaaTaiLisaa(aku1); kerat.getLkm() === 1;
     * kerat.korvaaTaiLisaa(aku2); kerat.getLkm() === 2;
     * Kera aku3 = aku1.clone();
     * aku3.setMaara("3");
     * Iterator<Kera> it = kerat.iterator();
     * it.next() == aku1 === true;
     * kerat.korvaaTaiLisaa(aku3); kerat.getLkm() === 2;
     * it = kerat.iterator();
     * Kera j0 = it.next();
     * j0 === aku3;
     * j0 == aku3 === true;
     * j0 == aku1 === false;
     * </pre>
     */
    public void korvaaTaiLisaa(Kera kera) throws SailoException {
        int id = kera.getTunnusNro();
        for (int i = 0; i < lkm; i++) {
            if ( alkiot[i].getTunnusNro() == id ) {
                alkiot[i] = kera;
                muutettu = true;
                return;
            }
        }
        lisaa(kera);
    }


    /**
     * Palauttaa lankojen määrän
     * @return lankojen lukumäärä
     */
    public int getLkm() {
        return lkm;
    }

    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     */
    public Kera anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }

    /**
     * @param tiednimi tiedoston nimi
     * @throws SailoException ongelmia avaamisessa
     */
    public void tallenna(String tiednimi) throws SailoException{
        File ftied = new File(tiednimi + "/langat.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(ftied,false))){
            for (int i = 0; i<getLkm();i++) {
                Kera kera = anna(i);
                fo.println(kera);
            }
        }catch (FileNotFoundException ex) {
            throw new SailoException("Tiedosto" + ftied.getName() + " ei aukea");
        }
    }
    

    /**
     * Lukee tiedostoa
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     * 
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.File;
     * 
     *  Kerat kerat = new Kerat();
     *  Kera lanka1 = new Kera(), lanka2 = new Kera();
     *  lanka1.tayta();
     *  lanka2.tayta();
     *  String hakemisto = "testikelmit";
     *  String tiedNimi = hakemisto;
     *  File ftied = new File(tiedNimi+".dat");
     *  File dir = new File(hakemisto);
     *  dir.mkdir();
     *  ftied.delete();
     *  kerat.lueTiedostosta(tiedNimi); 
     *  kerat.lisaa(lanka1);
     *  kerat.lisaa(lanka2);
     *  kerat.tallenna(tiedNimi);
     *  kerat = new Kerat();            // Poistetaan vanhat luomalla uusi
     *  kerat.lueTiedostosta(tiedNimi);  // johon ladataan tiedot tiedostosta.
     * </pre>
     */

    public void lueTiedostosta(String hakemisto) throws SailoException{
        String nimi = hakemisto + "/langat.dat";
        File ftied = new File(nimi);
        
        try (Scanner fi = new Scanner(new FileInputStream(ftied))) {
            while (fi.hasNext()) {
                String s = "";
                s = fi.nextLine();
                Kera kera = new Kera();
                kera.parse(s);
                lisaa(kera);
            }
            
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Ei saa luettua tiedostoa " + nimi);
        }
    }

    
    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonPerusNimi() {
        return tiedostonPerusNimi;
    }


    /**
     * Asettaa tiedoston perusnimen ilan tarkenninta
     * @param nimi tallennustiedoston perusnimi
     */
    public void setTiedostonPerusNimi(String nimi) {
        tiedostonPerusNimi = nimi;
    }


    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonNimi() {
        return getTiedostonPerusNimi() + ".dat";
    }


    /**
     * Palauttaa varakopiotiedoston nimen
     * @return varakopiotiedoston nimi
     */
    public String getBakNimi() {
        return tiedostonPerusNimi + ".bak";
    }

    
    
    /**
     * @param hakuehto hakuehto
     * @param k k
     * @return löytyneet
     */
    @SuppressWarnings("unused")
    public Collection<Kera> etsi(String hakuehto, int k) { 
        Collection<Kera> loytyneet = new ArrayList<Kera>(); 
        for (Kera kera : this) { 
            loytyneet.add(kera);  
        } 
        return loytyneet; 
    }

    
    /**
     * Testiohjelma jäsenistölle
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Kerat kerat = new Kerat();

        try {
            kerat.lueTiedostosta("langat");
        } catch (SailoException e) {
            System.err.println();
        }
        
        Kera lanka = new Kera();
        Kera lanka2 = new Kera();
        lanka.rekisteroi();
        lanka.tayta();
        lanka2.rekisteroi();
        lanka2.tayta();

        try {
            kerat.lisaa(lanka);
            kerat.lisaa(lanka2);
            

            System.out.println("============= Jäsenet testi =================");
            
            for (int i = 0; i < kerat.getLkm(); i++) {
                Kera kera = kerat.anna(i);
                System.out.println("Jäsen nro: " + i);
                kera.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
        
        try {
            kerat.tallenna("langat");
        } catch (SailoException e) {
            e.printStackTrace();
        }
        
    }

    /**
     * Luokka kerien iteroimiseksi.
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #PACKAGEIMPORT
     * #import java.util.*;
     * 
     * Kerat kerat = new Kerat();
     * Kera lanka1 = new Kera(), lanka2 = new Kera();
     * lanka1.rekisteroi(); lanka2.rekisteroi();
     *
     * kerat.lisaa(lanka1); 
     * kerat.lisaa(lanka2); 
     * kerat.lisaa(lanka1); 
     * 
     * StringBuffer ids = new StringBuffer(30);
     * for (Kera kera:kerat)   // Kokeillaan for-silmukan toimintaa
     *   ids.append(" "+kera.getTunnusNro());           
     * 
     * String tulos = " " + lanka1.getTunnusNro() + " " + lanka2.getTunnusNro() + " " + lanka1.getTunnusNro();
     * 
     * ids.toString() === tulos; 
     * 
     * ids = new StringBuffer(30);
     * for (Iterator<Kera>  i=kerat.iterator(); i.hasNext(); ) { // ja iteraattorin toimintaa
     *   Kera kera = i.next();
     *   ids.append(" "+kera.getTunnusNro());           
     * }
     * 
     * ids.toString() === tulos;
     * 
     * Iterator<Kera>  i=kerat.iterator();
     * i.next() == lanka1  === true;
     * i.next() == lanka2  === true;
     * i.next() == lanka1  === true;
     * 
     * i.next();  #THROWS NoSuchElementException
     *  
     * </pre>
     */

    public class KeratIterator implements Iterator<Kera> {
        private int kohdalla = 0;


        /**
         * Onko olemassa vielä seuraavaa jäsentä
         * @see java.util.Iterator#hasNext()
         * @return true jos on vielä jäseniä
         */
        @Override
        public boolean hasNext() {
            return kohdalla < getLkm();
        }


        /**
         * Annetaan seuraava kerä
         * @return seuraava kerä
         * @throws NoSuchElementException jos seuraava alkiota ei enää ole
         * @see java.util.Iterator#next()
         */
        @Override
        public Kera next() throws NoSuchElementException {
            if ( !hasNext() ) throw new NoSuchElementException("Ei oo");
            return anna(kohdalla++);
        }


        /**
         * Tuhoamista ei ole toteutettu
         * @throws UnsupportedOperationException aina
         * @see java.util.Iterator#remove()
         */
        @Override
        public void remove() throws UnsupportedOperationException {
            throw new UnsupportedOperationException("Me ei poisteta");
        }
    }

    @Override
    public Iterator<Kera> iterator() {
        return new KeratIterator();
    }
  
    


    

}
