package langat;

import java.io.*;
import java.util.Objects;

import fi.jyu.mit.ohj2.Mjonot;


/**
 * @author Käyttäjä
 * @version 6.4.2025
 *
 */
public class Kohteet {
    private int tunnusNro;
    private int keraNro;
    private String ala;

    private static int seuraavaNro = 1;


    /**
     * Alustetaan Kohde
     */
    public Kohteet() {
        // 
    }


    /**
     * Alustetaan käyttökohde.  
     * @param keraNro jäsenen viitenumero 
     */
    public Kohteet(int keraNro) {
        this.keraNro = keraNro;
    }


    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot Kerälle.
     * @param nro viite kerään
     */
    public void taytaPipo(int nro) {
        keraNro = nro;
        ala = "Pipo";
    }

    /**
     * Lisätään kerälle hyödynnyskohteita
     * @param nro kerän numero
     * @param comNro chooserin index
     */
    public void taytaKohd(int nro, int comNro) {
        keraNro = nro;
        String[] kohteet = {"","Pipo","Sukat/lapaset","Paita","Kassi","Vaatteet","Ompelu"};
        ala=kohteet[comNro];
    }
    
    /**
     * Tulostetaan kohteen tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(ala);
    }


    /**
     * Tulostetaan kohteen tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }


    /**
     * Antaa käyttökohteelle seuraavan rekisterinumeron.
     * @return harrastuksen uusi tunnus_nro
     * @example
     * <pre name="test">
     *   Kohteet pipo1 = new Kohteet();
     *   pipo1.getTunnusNro() === 0;
     *   pipo1.rekisteroi();
     *   Kohteet pipo2 = new Kohteet();
     *   pipo2.rekisteroi();
     *   int n1 = pipo1.getTunnusNro();
     *   int n2 = pipo2.getTunnusNro();
     *   n1 === n2-1;
     * </pre>
     */
    public int rekisteroi() {
        tunnusNro = seuraavaNro;
        seuraavaNro++;
        return tunnusNro;
    }


    /**
     * Palautetaan kohteen oma id
     * @return harrastuksen id
     */
    public int getTunnusNro() {
        return tunnusNro;
    }


    /**
     * Palautetaan mille kerälle kohde on voimassa
     * @return kerän id
     */
    public int getKeraNro() {
        return keraNro;
    }


    /**
     * hakee kohteet
     * @return kohde
     */
    public String getKohde() {
        return ala;
        
    }
    
    /**
     * Asettaa tunnusnumeron ja samalla varmistaa että
     * seuraava numero on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava tunnusnumero
     */
    private void setTunnusNro(int nr) {
        tunnusNro = nr;
        if ( tunnusNro >= seuraavaNro ) seuraavaNro = tunnusNro + 1;
    }


    /**
     * Palauttaa kohteen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return harrastus tolppaeroteltuna merkkijonona 
     * 
     */
    @Override
    public String toString() {
        return "" + getTunnusNro() + "|" + keraNro + "|" + ala;
    }

    /**
     * Palauttaa kohteen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @param kohd kohde
     * @return harrastus tolppaeroteltuna merkkijonona 
     */
    
    public String toString(@SuppressWarnings("unused") Kohteet kohd) {
        return "" + ala;
    }
    
    /**
     * Selvitää harrastuksen tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusnro.
     * @param rivi josta harrastuksen tiedot otetaan
     * @example
     * <pre name="test">
     *   Kohteet kohteet = new Kohteet();
     *   kohteet.parse("   2   |  2  |   Pipo  ");
     *   kohteet.getKeraNro() === 2;
     *   kohteet.toString()    === "2|2|Pipo";
     *   
     *   kohteet.rekisteroi();
     *   int n = kohteet.getTunnusNro();
     *   kohteet.parse(""+(n+20));
     *   kohteet.rekisteroi();
     *   kohteet.getTunnusNro() === n+20+1;
     *   kohteet.toString()     === "" + (n+20+1) + "|2|Pipo";
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setTunnusNro(Mjonot.erota(sb, '|', getTunnusNro()));
        keraNro = Mjonot.erota(sb, '|', keraNro);
        ala = Mjonot.erota(sb, '|', ala);
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true; 
        if (obj == null || getClass() != obj.getClass()) return false;

        Kohteet other = (Kohteet) obj;

        return this.tunnusNro == other.tunnusNro
            && this.keraNro == other.keraNro
            && Objects.equals(this.ala, other.ala); 
    }

    @Override
    public int hashCode() {
        return Objects.hash(tunnusNro, keraNro, ala);
    }
    
    /**
     * Testiohjelma Kohteelle.
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Kohteet kohd = new Kohteet();
        kohd.taytaPipo(2);
        kohd.tulosta(System.out);
    }

}
