/**
 * 
 */
package langat;

import java.io.File;
import java.util.Collection;
import java.util.List;

/**
 * @author Essi
 * @version 4.3.2025
 *
 */
public class Langat {
    
    private Kerat kerat = new Kerat();
    private Hyodynnys hyodynnys = new Hyodynnys();
    
    private String hakemisto = "langat";
    
    /**
     * Lisää lankoihin uuden langan
     * @param kera lisättävä kerä
     * @throws SailoException jos lisäystä ei voida tehdä
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Langat langat = new Langat();
     * Kera lanka1 = new Kera(), lanka2 = new Kera();
     * lanka1.rekisteroi(); lanka2.rekisteroi();
     * langat.getJasenia() === 0;
     * langat.lisaa(lanka1); langat.getJasenia() === 1;
     * langat.lisaa(lanka2); langat.getJasenia() === 2;
     * langat.lisaa(lanka1); langat.getJasenia() === 3;
     * langat.getJasenia() === 3;
     * langat.annaJasen(0) === lanka1;
     * langat.annaJasen(1) === lanka2;
     * langat.annaJasen(2) === lanka1;
     * langat.annaJasen(3) === lanka1; #THROWS IndexOutOfBoundsException 
     * langat.lisaa(lanka1); langat.getJasenia() === 4;
     * langat.lisaa(lanka1); langat.getJasenia() === 5;
     * langat.lisaa(lanka1);            
     * </pre>
     */
    public void lisaa(Kera kera) throws SailoException {
        kerat.lisaa(kera);
    }
    
    /**
     * Poistetaan kerä
     * @param paikka kerän paikka eli id
     */
    public void poistaKera(int paikka) {
        kerat.poistaKera(paikka);
    }


    /**
     * Poistetaan kohteet
     * @param paikka kerän id jolta poistetaan hyod.kohteet
     */
    public void poistaKohteet(int paikka) {
        hyodynnys.poistaKohteet(paikka);
        
    }
    /**
     * Listään uusi harrastus kerhoon
     * @param kohd lisättävä käyttökohde
     */
    public void lisaa(Kohteet kohd) {
        hyodynnys.lisaa(kohd);
    }

    /**
     * poistetaan hyod. kohde
     * @param kohd poistettava hyodynnys kohde
     */
    public void poista(Kohteet kohd) {
        hyodynnys.poista(kohd);
        
    }
    /**
     * @param lanka lanka jota tarkastellaan
     * @return hyödynnyskohde
     */
    public List<Kohteet> annaHyodynnysKohteet(Kera lanka) {
        return hyodynnys.annaHyodynnysKohteet(lanka.getTunnusNro());
    }

    /**
     * @param kera annettu kerä
     * @throws SailoException jos virhe
     */
    public void korvaaTaiLisaa(Kera kera) throws SailoException{
        kerat.korvaaTaiLisaa(kera);
    }
    
    /**
     * Palautaa lankojen jäsenmäärän
     * @return jäsenmäärä
     */
    public int getJasenia() {
        return kerat.getLkm();
    }

    /**
     * Palauttaa i:n jäsenen
     * @param i monesko jäsen palautetaan
     * @return viite i:teen jäseneen
     * @throws IndexOutOfBoundsException jos i väärin
     */
    public Kera annaJasen(int i) throws IndexOutOfBoundsException {
        return kerat.anna(i);
    }

    /**
     * Lukee kerhon tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     * 
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     * 
     *  Langat langat = new Langat();
     *  
     *  Kera aku1 = new Kera(); aku1.tayta(); aku1.rekisteroi();
     *  Kera aku2 = new Kera(); aku2.tayta(); aku2.rekisteroi();
     *  Kohteet pitsi21 = new Kohteet(); pitsi21.taytaPipo(aku2.getTunnusNro());
     *  Kohteet pitsi11 = new Kohteet(); pitsi11.taytaPipo(aku1.getTunnusNro()); pitsi11.rekisteroi();
     *  Kohteet pitsi22 = new Kohteet(); pitsi22.taytaPipo(aku2.getTunnusNro()); pitsi22.rekisteroi();
     *  Kohteet pitsi12 = new Kohteet(); pitsi12.taytaPipo(aku1.getTunnusNro()); pitsi12.rekisteroi();
     *  Kohteet pitsi23 = new Kohteet(); pitsi23.taytaPipo(aku2.getTunnusNro()); pitsi23.rekisteroi();
     *   
     *  String hakemisto = "testikelmit";
     *  File dir = new File(hakemisto);
     *  File ftied  = new File(hakemisto+"/nimet.dat");
     *  File fhtied = new File(hakemisto+"/harrastukset.dat");
     *  dir.mkdir();  
     *  ftied.delete();
     *  fhtied.delete();
     *  langat.lueTiedostosta(hakemisto); 
     *  langat.lisaa(aku1);
     *  langat.lisaa(aku2);
     *  langat.lisaa(pitsi21);
     *  langat.lisaa(pitsi11);
     *  langat.lisaa(pitsi22);
     *  langat.lisaa(pitsi12);
     *  langat.lisaa(pitsi23);
     *  langat.tallenna();
     *  langat = new Langat();
     *  langat.lueTiedostosta(hakemisto);
     * </pre>
     */

    public void lueTiedostosta(String nimi) throws SailoException {
        File dir = new File(nimi);
        dir.mkdir();
        kerat = new Kerat(); // jos luetaan olemassa olevaan niin helpoin tyhjentää näin
        hyodynnys = new Hyodynnys();

        hakemisto = nimi;
        kerat.lueTiedostosta(nimi);
        hyodynnys.lueTiedostosta(nimi);
    }

    /**
     * @throws SailoException virhe tapauksissa
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            kerat.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            hyodynnys.tallenna(hakemisto);
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }

    /**
     * @param hakuehto ehto
     * @param k k
     * @return etsi
     * @throws SailoException jos vikaa 
     */
    public Collection<Kera> etsi(String hakuehto, int k) throws SailoException { 
        return kerat.etsi(hakuehto, k); 
    } 

    
    /**
     * Testiohjelma kerholle
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Langat langat = new Langat();

        try {
            // langat.lueTiedostosta("langat");
            langat.lueTiedostosta("koelangat");
        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
        try {

            Kera lanka = new Kera();
            Kera lanka2 = new Kera();
            lanka.rekisteroi();
            lanka.tayta();
            lanka2.rekisteroi();
            lanka2.tayta();
            
            langat.lisaa(lanka);
            langat.lisaa(lanka2);
            

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
        try {

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < langat.getJasenia(); i++) {
                Kera kera = langat.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                kera.tulosta(System.out);
            }
            
         langat.tallenna();    

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
    }






}
